#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
# from datetime import date


"""OpenDataLab python SDK version info."""

__url__ = "http://opendatalab-test.shlab.tech" # "https://opendatalab.com"
__version__ = "0.0.3b02"
__svc__ = '1.9'
odl_clientId = "ypkl8bwo0eb5ao1b96no"  # "kmz3bkwzlaa3wrq8pvwa"
uaa_url_prefix = "http://uaa-staging.mai4u.com/gw/uaa-be"  # "https://sso.openxlab.org.cn/gw/uaa-be"
